local AceOO = AceLibrary("AceOO-2.0")

if not Bartender3.Class then Bartender3.Class = {} end
Bartender3.Class.Bar = AceOO.Class("AceEvent-2.0")
Bartender3.Class.ActionBar = AceOO.Class(Bartender3.Class.Bar)
Bartender3.Class.SpecialBar = AceOO.Class(Bartender3.Class.Bar)

function Bartender3.Class.Bar.prototype:init(id, config, mainbar)
	Bartender3.Class.Bar.super.prototype.init(self)
	self.id = id
	self.mainbar = mainbar
	self.config = config
	self.barname = "Bar "..self.id
	
	self:CreateBarFrame()
end

function Bartender3.Class.Bar.prototype:ChangeProfile(config)
	self.config = config
	self:RefreshLayout()
end

function Bartender3.Class.Bar.prototype:CreateBarFrame()
	local name = "BT3Bar"..self.id
	local class = Bartender3.playerclass
	local frame = CreateFrame("Frame", name, UIParent, self.mainbar and "SecureStateDriverTemplate" or "SecureStateHeaderTemplate")
	if self.mainbar then
		if class == "WARRIOR" or class == "DRUID" or class == "ROGUE" then
			local form = GetShapeshiftForm(true)
			frame:SetAttribute("statebutton","0:S0;1:S1;2:S2;3:S3;4:S4;5:S5;6:S6")
			frame:SetAttribute("statemap-stance", "$input")
			if class == "DRUID" then
				--[[ I set all forms which do not need a special page to state 0, so i can have a cleaner layout there :)
					
					"0" - Caster Form
					"1" - Bear Form
					"3" - Cat Form (non prowl)
					"4" - Cat Form (prowl)
					
					All other states are unused and free for later usage
				]]
				frame:SetAttribute("statemap-stance-0", "0") -- caster
				frame:SetAttribute("statemap-stance-1", "1") -- bear
				frame:SetAttribute("statemap-stance-2", "0") -- aquatic
				frame:SetAttribute("statemap-stance-3", "3") -- cat
				frame:SetAttribute("statemap-stance-4", "0") -- travel
				frame:SetAttribute("statemap-stance-5", "0") -- moonkin/tree
				frame:SetAttribute("statemap-stance-6", "0") -- flying form
				frame:SetAttribute("statemap-stealth-1", "3:4") -- stealthed / prowl
				frame:SetAttribute("statemap-stealth-0", "4:3") -- unstealthed
				
				if class == "DRUID" then
					if form ~= 1 and form ~= 3 then form = 0
					elseif form == 3 and IsStealthed() then form = 4 end
				end
			end
			frame:SetAttribute("state", tostring(form))
		end
	end
	frame:EnableMouse(false)
	frame:SetMovable(true)
	frame:RegisterForDrag("LeftButton")
	frame:SetWidth(10)
	frame:SetHeight(10)
	frame:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16, edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16, insets = {left = 5, right = 3, top = 3, bottom = 5},})
	frame:SetBackdropColor(0, 0, 0, 0)
	frame:SetBackdropBorderColor(0.5, 0.5, 0, 0)
	frame:ClearAllPoints()
	frame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	frame.Text = frame:CreateFontString(nil, "ARTWORK")
	frame.Text:SetFontObject(GameFontNormal)
	frame.Text:SetText()
	frame.Text:Show()
	frame.Text:ClearAllPoints()
	frame.Text:SetPoint("CENTER", frame, "CENTER",0,0)
	frame:SetPoint("CENTER")
	frame:Show()
	self.frame = frame
end

function Bartender3.Class.Bar.prototype:UnlockFrames()
	local frame = self.frame
	frame:EnableMouse(true)
	frame:SetScript("OnEnter", function() this:SetBackdropBorderColor(0.5, 0.5, 0, 1) end)
	frame:SetScript("OnLeave", function() this:SetBackdropBorderColor(0, 0, 0, 0) end)
	frame:SetScript("OnDragStart", function()
		this:StartMoving()
		this:SetBackdropBorderColor(0, 0, 0, 0)
	end)
	frame:SetScript("OnDragStop", function()
		this:StopMovingOrSizing()
		self:SavePosition()
	end)
--[[	frame:SetScript("OnClick", function()
		self.config.Hide = not self.config.Hide
		if self.config.Hide then
			this:SetBackdropColor(1, 0, 0, 0.5)
		else
			this:SetBackdropColor(0, 1, 0, 0.5)
		end
	end) ]]
	if self.config.Hide then
		frame:SetBackdropColor(1, 0, 0, 0.5)
	else
		frame:SetBackdropColor(0, 1, 0, 0.5)
	end
	frame:SetFrameLevel(3)
	for i,v in pairs(self.buttons) do v:SetAlpha(0.5) v:SetFrameLevel(2) end
	frame.Text:SetText(self.barname)
end

function Bartender3.Class.Bar.prototype:LockFrames()
	local frame = self.frame
	frame:EnableMouse(false)
	frame:SetScript("OnEnter", nil)
	frame:SetScript("OnLeave", nil)
	frame:SetScript("OnDragStart", nil)
	frame:SetScript("OnDragStop", nil)
	--frame:SetScript("OnClick", nil)
	frame:SetBackdropColor(0, 0, 0, 0)
	frame:SetBackdropBorderColor(0, 0, 0, 0)
	frame.Text:SetText("")
	frame:SetFrameLevel(1)
	if self.config.Hide then frame:Hide() else frame:Show() end
	for i,v in pairs(self.buttons) do v:SetAlpha(self.config.Alpha) v:SetFrameLevel(2) end
end

function Bartender3.Class.Bar.prototype:RefreshLayout()
	self:LoadPosition()
	self:RefreshButtonLayout()
	self:ButtonZoomCheck()
end

function Bartender3.Class.Bar.prototype:LoadPosition()
	local x,y,s = self.config.PosX or 0, self.config.PosY or 0, self.frame:GetEffectiveScale()
	x,y = x/s, y/s
	self.frame:ClearAllPoints()
	self.frame:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", x, y)
end

function Bartender3.Class.Bar.prototype:SavePosition()
	local frame = self.frame
	local x,y = frame:GetLeft(), frame:GetBottom()
	local s = frame:GetEffectiveScale()
	x,y = x*s,y*s
	self.config.PosX = x
	self.config.PosY = y
end

function Bartender3.Class.Bar.prototype:RefreshButtonLayout()
	local Rows = self.config.Rows
	local ButtonPerRow = math.floor(#self.buttons / Rows) -- just a precaution
	local Padding = self.config.Padding
	self.frame:SetWidth(self.buttonsize * ButtonPerRow + ((ButtonPerRow - 1) * Padding) + 8)
	self.frame:SetHeight(self.buttonsize * Rows + ((Rows - 1) * Padding) + 8)
	for i=1,#self.buttons do
		local button = self.buttons[i]
		button:ClearAllPoints()
		if i > 1 and ((i-1) % ButtonPerRow) == 0 then
			button:SetPoint("TOPLEFT", self.buttons[i-ButtonPerRow], "BOTTOMLEFT", 0, -Padding)
		elseif i > 1 then
			button:SetPoint("TOPLEFT", self.buttons[i-1], "TOPRIGHT", Padding, 0)
		else
			button:SetPoint("TOPLEFT", self.frame, "TOPLEFT", 5, -3)
		end
	end
end

function Bartender3.Class.Bar.prototype:ButtonZoomCheck()
	local Zoomed = Bartender3.db.profile.Zoomed
	if Zoomed then
		for i,v in pairs(self.buttons) do v.icon:SetTexCoord(0.07,0.93,0.07,0.93) end
	else
		for i,v in pairs(self.buttons) do v.icon:SetTexCoord(0,1,0,1) end
	end
end

--[[
	Action Bars
	
	ActionBar is the abstract class for all of the 10 actionbars
	self.buttons holds the actual frames
	self.buttonobjects holds the ButtonClass objects
]]

function Bartender3.Class.ActionBar.prototype:init(id, config, numbuttons, mainbar)
	Bartender3.Class.ActionBar.super.prototype.init(self, id, config, mainbar)
	self.numbuttons = numbuttons
	self.buttonsize = 36
	self:CreateButtons()
	self:RefreshLayout()
end

function Bartender3.Class.ActionBar.prototype:CreateButtons()
	if self.buttons or self.buttonobjects then return end
	self.buttons, self.buttonobjects = {}, {}
	for i=1, self.numbuttons do
		local button = Bartender3:GetFreeButton(self)
		table.insert(self.buttons, button.frame)
		table.insert(self.buttonobjects, button)
	end
end

--[[
	Special Bars
	
	Special Bars are an abstraction layer for the MicroMenu/Bags/Stance/Pet Bar
	The table self.buttons holds the actual frames of the buttons.
]]

function Bartender3.Class.SpecialBar.prototype:init(bartype, barname, config)
	Bartender3.Class.ActionBar.super.prototype.init(self, bartype, config, mainbar)
	self.bartype = bartype
	self.barname = barname
	self:AssignFrames(bartype)
	self:RefreshLayout()
end

function Bartender3.Class.SpecialBar.prototype:AssignFrames(bartype)
	if self.buttons then return end
	self.buttons = {}
	if not bartype then bartype = self.bartype end
	if ( bartype == "STANCE" ) then
		for i=1,10 do 
			local button = getglobal("ShapeshiftButton"..i)
			button:SetParent(self.frame)
			button:SetNormalTexture("")
			self.buttonsize = button:GetWidth()
			
			button.icon = getglobal("ShapeshiftButton"..i.."Icon")
			
			table.insert(self.buttons, button) 
		end
	end
end

